/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.apache.commons.io.FileUtils;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Unit test about the legal information that the archive must contain
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class LegalIT {

    private final static String PROPERTY_NAME_ARCHIVE_FILE = "archive-file";

    private static File ARCHIVE_FILE = null;

    private final static String PROPERTY_NAME_ARCHIVE_BASE_DIRECTORY = "archive-base-directory";

    private static File ARCHIVE_BASE_DIRECTORY = null;

    @BeforeClass
    public static void setup() {
        final String archiveFileStr = System.getProperty(PROPERTY_NAME_ARCHIVE_FILE);
        assertNotNull("System property '" + PROPERTY_NAME_ARCHIVE_FILE + "' not set.", archiveFileStr);
        ARCHIVE_FILE = new File(archiveFileStr);
        assertTrue("The archive file '" + ARCHIVE_FILE.getAbsolutePath() + "' does not exist", ARCHIVE_FILE.exists());
        assertTrue("The archive file '" + ARCHIVE_FILE.getAbsolutePath() + "' is not a file", ARCHIVE_FILE.isFile());

        final String archiveBaseDirectoryStr = System.getProperty(PROPERTY_NAME_ARCHIVE_BASE_DIRECTORY);
        assertNotNull("System property '" + PROPERTY_NAME_ARCHIVE_BASE_DIRECTORY + "' not set.",
                archiveBaseDirectoryStr);
        ARCHIVE_BASE_DIRECTORY = new File(archiveBaseDirectoryStr);
        assertTrue("The archive base directory '" + ARCHIVE_BASE_DIRECTORY.getAbsolutePath() + "' does not exist",
                ARCHIVE_BASE_DIRECTORY.exists());
        assertTrue("The archive base directory '" + ARCHIVE_BASE_DIRECTORY.getAbsolutePath() + "' is not a directory",
                ARCHIVE_BASE_DIRECTORY.isDirectory());
    }

    /**
     * <p>
     * Verify that a file 'LICENSE' exists.
     * </p>
     */
    @Test
    public void licenseFile() throws IOException {
        
        final File licenseFile = new File(ARCHIVE_BASE_DIRECTORY, "LICENSE");
        assertTrue("'LICENSE' file missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, licenseFile));
        assertTrue("Too short file 'LICENSE'", FileUtils.sizeOf(licenseFile) > 20000);
    }

    /**
     * <p>
     * Verify that a file 'THIRD-PARTY' exists.
     * </p>
     */
    @Test
    public void thirdPartyFile() throws IOException {

        final File thirdPartyFile = new File(ARCHIVE_BASE_DIRECTORY, "THIRD-PARTY");
        assertTrue("'THIRD-PARTY' file missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, thirdPartyFile));
        assertTrue("Too short file 'THIRD-PARTY'", FileUtils.sizeOf(thirdPartyFile) < 256);
    }

    /**
     * <p>
     * Verify that no license files about third party products is packaged.
     * </p>
     */
    @Test
    public void thirdPartyLicenseFiles() throws IOException {

        final File thirdPartyLIcenseFiles = new File(ARCHIVE_BASE_DIRECTORY, "licenses");
        assertFalse("3PP licenses directory exists",
                FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, thirdPartyLIcenseFiles));
    }

    /**
     * <p>
     * Verify that a file 'COPYRIGHT' exists.
     * </p>
     */
    @Test
    public void copyrightFile() throws IOException {

        final File copyrightFile = new File(ARCHIVE_BASE_DIRECTORY, "COPYRIGHT");
        assertTrue("'COPYRIGHT' file missing", FileUtils.directoryContains(ARCHIVE_BASE_DIRECTORY, copyrightFile));
        assertTrue("Too short file 'COPYRIGHT'", FileUtils.sizeOf(copyrightFile) > 500);
        assertTrue("Too long file 'COPYRIGHT'", FileUtils.sizeOf(copyrightFile) < 1256);
    }

    /**
     * <p>
     * Verify that all JBI components contain legal files.
     * </p>
     */
    @Test
    public void legalContentForJbiArchives() throws IOException {

        try (final InputStream inArchive = new FileInputStream(ARCHIVE_FILE);
                final ZipInputStream zipArchive = new ZipInputStream(inArchive)) {
            ZipEntry zipArchiveEntry;
            while ((zipArchiveEntry = zipArchive.getNextEntry()) != null) {
                if (!zipArchiveEntry.isDirectory()) {
                    final String zipArchiveEntryName = zipArchiveEntry.getName();
                    // TODO: Fix PETALSSLJBOSS-1 and remove the hack on 'petals-sl-jboss-6.0.0-M2'
                    if (zipArchiveEntryName.contains("esb-components/") || (zipArchiveEntryName.contains("esb-sls/")
                            && !zipArchiveEntryName.contains("petals-sl-jboss-6.0.0.M2"))) {

                        try (final ZipInputStream zipComponent = new ZipInputStream(new FilterInputStream(zipArchive) {
                            @Override
                            public void close() throws IOException {
                                // do not close, we do it a the end after reading all the entries!
                            }
                        })) {
                            boolean licenseEntryOk = false;
                            boolean copyrightEntryOk = false;
                            boolean thirdpartyEntryOk = false;
                            boolean thirdpartyLicensesEntryOk = false;
                            ZipEntry zipComponentEntry;
                            while ((zipComponentEntry = zipComponent.getNextEntry()) != null) {
                                if (!zipComponentEntry.isDirectory()) {
                                    final String zipComponentEntryName = zipComponentEntry.getName();
                                    if (zipComponentEntryName.equals("/META-INF/legal/LICENSE")) {
                                        licenseEntryOk = true;
                                    } else if (zipComponentEntryName.equals("/META-INF/legal/COPYRIGHT")) {
                                        copyrightEntryOk = true;
                                    } else if (zipComponentEntryName.equals("/META-INF/legal/THIRD-PARTY")) {
                                        thirdpartyEntryOk = true;
                                    } else if (zipComponentEntryName.startsWith("/META-INF/legal/licenses/")) {
                                        thirdpartyLicensesEntryOk = true;
                                    } else if (zipArchiveEntryName.contains("samples/")) {
                                        // Samples does not contains the file THIRD-PARTY and 3PP license files
                                        thirdpartyLicensesEntryOk = true;
                                        thirdpartyEntryOk = true;
                                    }
                                }
                            }
                            assertTrue("'LICENSE' missing in JBI archive: " + zipArchiveEntryName, licenseEntryOk);
                            assertTrue("'COPYRIGHT' missing in JBI archive: " + zipArchiveEntryName,
                                    copyrightEntryOk);
                            assertTrue("'THIRD-PARTY' missing in JBI archive: " + zipArchiveEntryName,
                                    thirdpartyEntryOk);
                            assertTrue("Third party licenses missing in JBI archive: " + zipArchiveEntryName,
                                    thirdpartyLicensesEntryOk);
                        }
                    }
                }
            }
        }
    }
}
